import asyncio
import collections.abc
import datetime
import json
import math
import multiprocessing
import operator
import os
import pathlib
import platform
import pprint
import queue
import random
import re
import io
import shutil
import subprocess
import sys
import threading
import time
import timeit
import urllib.request
import urllib.parse
import uuid
import warnings
import webbrowser
import zoneinfo
import textwrap
import array


''' platform '''
WINDOWS = platform.system() == 'Windows'
LINUX = not WINDOWS



if __name__ == "__main__":
    print_colorize("Anda menjalankan module pypipr", color=colorama.Fore.RED)
